@@ -23,6 +23,8 @@
 #include "gailbutton.h"
 #include <libgail-util/gailmisc.h>
 
+#define GAIL_BUTTON_ATTACHED_MENUS "gtk-attached-menus"
+
 static void                  gail_button_class_init       (GailButtonClass *klass);
 static void                  gail_button_object_init      (GailButton      *button);
 
@@ -136,6 +138,10 @@ static gint                  get_n_labels_from_button   (GtkWidget      *button)
 static void                  set_role_for_button        (AtkObject      *accessible,
                                                          GtkWidget      *button);
 
+static gint                  get_n_attached_menus       (GtkWidget      *widget);
+static GtkWidget*            get_nth_attached_menu      (GtkWidget      *widget,
+                                                         gint           index);
+
 static GailContainer* parent_class = NULL;
 
 GType
@@ -858,10 +864,11 @@ gail_button_get_n_children (AtkObject* obj)
     return 0;
 
   /*
-   * Check whether we have an attached menu for PanelMenuButton
+   * Check whether we have an attached menus for PanelMenuButton
    */
-  if (g_object_get_data (G_OBJECT (widget), "gtk-attached-menu"))
-    return 1;
+  n_children = get_n_attached_menus (widget);
+  if (n_children > 0)
+    return n_children;
 
   n_children = get_n_labels_from_button (widget);
   if (n_children <= 1)
@@ -887,10 +894,15 @@ gail_button_ref_child (AtkObject *obj,
      */
     return NULL;
 
-  if (i == 0)
-    child_widget = g_object_get_data (G_OBJECT (widget), "gtk-attached-menu");
+  if (i >= gail_button_get_n_children (obj))
+    return NULL;
+
+  if (get_n_attached_menus (widget) > 0)
+  {
+    child_widget = get_nth_attached_menu (widget, i);
+  }
   else
-    child_widget = NULL;
+    child_widget = NULL;    
 
   if (!child_widget) 
     {
@@ -1673,3 +1685,37 @@ set_role_for_button (AtkObject *accessible,
   accessible->role =  role;
 }
 
+static gint
+get_n_attached_menus (GtkWidget  *widget)
+{
+  GList *list_menus;
+  
+  if (widget == NULL)
+    return 0;
+
+  list_menus = g_object_get_data (G_OBJECT (widget), GAIL_BUTTON_ATTACHED_MENUS);
+  if (list_menus == NULL)
+    return 0;
+
+  return g_list_length (list_menus);
+}
+
+static GtkWidget*
+get_nth_attached_menu (GtkWidget  *widget,
+                       gint       index)
+{
+  GtkWidget *attached_menu;
+  GList *list_menus;
+
+  if (widget == NULL)
+    return NULL;
+
+  list_menus = g_object_get_data (G_OBJECT (widget), GAIL_BUTTON_ATTACHED_MENUS);
+  if (list_menus == NULL ||
+      index >= g_list_length (list_menus))
+    return NULL;
+
+  attached_menu = (GtkWidget *) g_list_nth_data (list_menus, index);
+
+  return attached_menu;
+}
