@@ -231,6 +231,7 @@ gail_item_finalize (GObject *object)
   if (item->text)
     {
       g_free (item->text);
+      item->text = NULL;
     }
   G_OBJECT_CLASS (parent_class)->finalize (object);
 }
@@ -300,7 +301,6 @@ gail_item_get_name (AtkObject *obj)
 
                       model = gtk_combo_box_get_model (GTK_COMBO_BOX (parent));                       
                       item = GAIL_ITEM (obj);
-                      g_free (item->text);
                       if (gtk_tree_model_iter_nth_child (model, &iter, NULL, index))
                         {
                           n_columns = gtk_tree_model_get_n_columns (model);
@@ -311,7 +311,8 @@ gail_item_get_name (AtkObject *obj)
                                gtk_tree_model_get_value (model, &iter, i, &value);
                                if (G_VALUE_HOLDS_STRING (&value))
                                  {
-                                   item->text =  (gchar *) g_value_get_string (&value);
+				   g_free (item->text);
+                                   item->text =  (gchar *) g_value_dup_string (&value);
                                    break;
                                  }
                             }
