@@ -21,6 +21,7 @@
 #include <gtk/gtk.h>
 #include "gailnotebookpage.h"
 #include <libgail-util/gailmisc.h>
+#include "gail-private-macros.h"
 
 static void      gail_notebook_page_class_init      (GailNotebookPageClass     *klass);
 
@@ -392,7 +393,7 @@ gail_notebook_page_ref_child (AtkObject *accessible,
     return NULL;
 
   child = gtk_notebook_get_nth_page (page->notebook, page->index);
-  g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
+  gail_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
    
   child_obj = gtk_widget_get_accessible (child);
   g_object_ref (child_obj);
@@ -433,7 +434,7 @@ gail_notebook_page_ref_state_set (AtkObject *accessible)
       AtkObject *child;
 
       child = atk_object_ref_accessible_child (accessible, 0);
-      g_return_val_if_fail (child, state_set);
+      gail_return_val_if_fail (child, state_set);
 
       merged_state_set = state_set;
       state_set = atk_object_ref_state_set (child);
@@ -504,7 +505,7 @@ gail_notebook_page_get_extents (AtkComponent *component,
       *height = 0;
 
       child = atk_object_ref_accessible_child (ATK_OBJECT (component), 0);
-      g_return_if_fail (child);
+      gail_return_if_fail (child);
 
       atk_component_get_position (ATK_COMPONENT (child), x, y, coord_type);
       g_object_unref (child);
@@ -825,6 +826,7 @@ get_label_from_notebook_page (GailNotebookPage *page)
     return NULL;
 
   child = gtk_notebook_get_nth_page (notebook, page->index);
+  if (child == NULL) return NULL;
   g_return_val_if_fail (GTK_IS_WIDGET (child), NULL);
 
   child = gtk_notebook_get_tab_label (notebook, child);
