@@ -22,6 +22,9 @@
 #include "gailwidget.h"
 #include "gailnotebookpage.h"
 #include "gailcanvaswidget.h"
+#include "gail-private-macros.h"
+
+extern GtkWidget *focus_widget;
 
 static void gail_widget_class_init (GailWidgetClass *klass);
 
@@ -77,14 +80,6 @@ static gboolean gail_widget_set_size       (AtkComponent    *component,
                                             gint            width,
                                             gint            height);
 
-#if 0
-/*
- * We will get the parent from the widget structure rather than
- * use this function
- */
-static GtkWidget* _gail_widget_get_parent        (GtkWidget     *widget);
-#endif
-
 static gint       gail_widget_map_gtk            (GtkWidget     *widget);
 static void       gail_widget_real_notify_gtk    (GObject       *obj,
                                                   GParamSpec    *pspec);
@@ -94,6 +89,8 @@ static gboolean   gail_widget_focus_gtk          (GtkWidget     *widget,
                                                   GdkEventFocus *event);
 static gboolean   gail_widget_real_focus_gtk     (GtkWidget     *widget,
                                                   GdkEventFocus *event);
+static void       gail_widget_size_allocate_gtk  (GtkWidget     *widget,
+                                                  GtkAllocation *allocation);
 
 static void       gail_widget_focus_event        (AtkObject     *obj,
                                                   gboolean      focus_in);
@@ -192,7 +189,11 @@ gail_widget_real_initialize (AtkObject *obj,
   g_signal_connect (widget,
                     "notify",
                     G_CALLBACK (gail_widget_notify_gtk),
-                    widget);
+                    NULL);
+  g_signal_connect (widget,
+                    "size_allocate",
+                    G_CALLBACK (gail_widget_size_allocate_gtk),
+                    NULL);
   atk_component_add_focus_handler (ATK_COMPONENT (accessible),
                                    gail_widget_focus_event);
   /*
@@ -269,7 +270,7 @@ gail_widget_get_description (AtkObject *accessible)
       GtkAccessible *obj = GTK_ACCESSIBLE (accessible);
       GtkTooltipsData *data;
 
-      g_return_val_if_fail (obj, NULL);
+      gail_return_val_if_fail (obj, NULL);
 
       if (obj->widget == NULL)
         /*
@@ -277,7 +278,7 @@ gail_widget_get_description (AtkObject *accessible)
          */
         return NULL;
  
-      g_return_val_if_fail (GTK_WIDGET (obj->widget), NULL);
+      gail_return_val_if_fail (GTK_WIDGET (obj->widget), NULL);
     
       data = gtk_tooltips_data_get (obj->widget);
       if (data == NULL)
@@ -306,12 +307,41 @@ gail_widget_get_parent (AtkObject *accessible)
          * State is defunct
          */
         return NULL;
-      g_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
+      gail_return_val_if_fail (GTK_IS_WIDGET (widget), NULL);
 
       parent_widget = widget->parent;
       if (parent_widget == NULL)
         return NULL;
 
+      /*
+       * For a widget whose parent is a GtkNoteBook, we return the
+       * accessible object corresponding the GtkNotebookPage containing
+       * the widget as the accessible parent.
+       */
+      if (GTK_IS_NOTEBOOK (parent_widget))
+        {
+          gint page_num;
+          GtkWidget *child;
+          GtkNotebook *notebook;
+
+          page_num = 0;
+          notebook = GTK_NOTEBOOK (parent_widget);
+          while (TRUE)
+            {
+              child = gtk_notebook_get_nth_page (notebook, page_num);
+              if (!child)
+                break;
+              if (child == widget)
+                {
+                  parent = gtk_widget_get_accessible (parent_widget);
+                  parent = atk_object_ref_accessible_child (parent, page_num);
+                  g_object_unref (parent);
+                  return parent;
+                }
+              page_num++;
+            }
+        }
+
       parent = gtk_widget_get_accessible (parent_widget);
     }
   return parent;
@@ -371,7 +401,7 @@ gail_widget_ref_relation_set (AtkObject *obj)
   AtkObject *array[1];
   AtkRelation* relation;
 
-  g_return_val_if_fail (GAIL_IS_WIDGET (obj), NULL);
+  gail_return_val_if_fail (GAIL_IS_WIDGET (obj), NULL);
 
   widget = GTK_ACCESSIBLE (obj)->widget;
   if (widget == NULL)
@@ -392,29 +422,69 @@ gail_widget_ref_relation_set (AtkObject *obj)
   if (!atk_relation_set_contains (relation_set, ATK_RELATION_LABELLED_BY))
     {
       label = find_label (widget);
-      if (label == NULL && GTK_IS_BUTTON (widget))
-        /*
-         * Handle the case where GnomeIconEntry is the mnemonic widget.
-         * The GtkButton which is a grandchild of the GnomeIconEntry
-         * should really be the mnemonic widget. See bug #133967.
-         */
+      if (label == NULL)
         {
-          GtkWidget *temp_widget;
-
-          temp_widget = gtk_widget_get_parent (widget);
-
-          if (GTK_IS_ALIGNMENT (temp_widget))
+          if (GTK_IS_BUTTON (widget))
+            /*
+             * Handle the case where GnomeIconEntry is the mnemonic widget.
+             * The GtkButton which is a grandchild of the GnomeIconEntry
+             * should really be the mnemonic widget. See bug #133967.
+             */
             {
-              temp_widget = gtk_widget_get_parent (temp_widget);
-              if (GTK_IS_BOX (temp_widget))
+              GtkWidget *temp_widget;
+
+              temp_widget = gtk_widget_get_parent (widget);
+
+              if (GTK_IS_ALIGNMENT (temp_widget))
+                {
+                  temp_widget = gtk_widget_get_parent (temp_widget);
+                  if (GTK_IS_BOX (temp_widget))
+                    {
+                      label = find_label (temp_widget);
+                 
+                      if (!label)
+                        label = find_label (gtk_widget_get_parent (temp_widget));
+                    }
+                }
+            }
+          else if (GTK_IS_COMBO (widget))
+            /*
+             * Handle the case when GnomeFileEntry is the mnemonic widget.
+             * The GnomeEntry which is a grandchild of the GnomeFileEntry
+             * should be the mnemonic widget. See bug #137584.
+             */
+            {
+              GtkWidget *temp_widget;
+
+              temp_widget = gtk_widget_get_parent (widget);
+
+              if (GTK_IS_HBOX (temp_widget))
+                {
+                  temp_widget = gtk_widget_get_parent (temp_widget);
+                  if (GTK_IS_BOX (temp_widget))
+                    {
+                      label = find_label (temp_widget);
+                    }
+                }
+            }
+          else if (GTK_IS_COMBO_BOX (widget))
+            /*
+             * Handle the case when GtkFileChooserButton is the mnemonic
+             * widget.  The GtkComboBox which is a child of the
+             * GtkFileChooserButton should be the mnemonic widget.
+             * See bug #359843.
+             */
+            {
+              GtkWidget *temp_widget;
+
+              temp_widget = gtk_widget_get_parent (widget);
+              if (GTK_IS_HBOX (temp_widget))
                 {
                   label = find_label (temp_widget);
-                 
-                  if (!label)
-                    label = find_label (gtk_widget_get_parent (temp_widget));
                 }
             }
         }
+
       if (label)
         {
 	  array [0] = gtk_widget_get_accessible (label);
@@ -471,22 +541,23 @@ gail_widget_ref_state_set (AtkObject *accessible)
        * GailWidget data structure so we can determine whether the value has 
        * changed.
        */
-      if (gail_widget_on_screen (widget))
+      if (GTK_WIDGET_VISIBLE (widget))
         {
-          if (GTK_WIDGET_VISIBLE (widget))
+          atk_state_set_add_state (state_set, ATK_STATE_VISIBLE);
+          if (gail_widget_on_screen (widget) &&
+              GTK_WIDGET_MAPPED (widget))
             {
-              atk_state_set_add_state (state_set, ATK_STATE_VISIBLE);
-
-              if (GTK_WIDGET_MAPPED (widget))
-                {
-                  atk_state_set_add_state (state_set, ATK_STATE_SHOWING);
-                }
+              atk_state_set_add_state (state_set, ATK_STATE_SHOWING);
             }
         }
   
-      if (GTK_WIDGET_HAS_FOCUS (widget))
+      if (GTK_WIDGET_HAS_FOCUS (widget) && (widget == focus_widget))
         {
-          atk_state_set_add_state (state_set, ATK_STATE_FOCUSED);
+          AtkObject *focus_obj;
+
+          focus_obj = g_object_get_data (G_OBJECT (accessible), "gail-focus-object");
+          if (focus_obj == NULL)
+            atk_state_set_add_state (state_set, ATK_STATE_FOCUSED);
         }
     }
   return state_set;
@@ -538,11 +609,11 @@ gail_widget_get_index_in_parent (AtkObject *accessible)
         }
     }
 
-  g_return_val_if_fail (GTK_IS_WIDGET (widget), -1);
+  gail_return_val_if_fail (GTK_IS_WIDGET (widget), -1);
   parent_widget = widget->parent;
   if (parent_widget == NULL)
     return -1;
-  g_return_val_if_fail (GTK_IS_CONTAINER (parent_widget), -1);
+  gail_return_val_if_fail (GTK_IS_CONTAINER (parent_widget), -1);
 
   children = gtk_container_get_children (GTK_CONTAINER (parent_widget));
 
@@ -617,7 +688,7 @@ gail_widget_get_extents (AtkComponent   *component,
      */
     return;
 
-  g_return_if_fail (GTK_IS_WIDGET (widget));
+  gail_return_if_fail (GTK_IS_WIDGET (widget));
 
   *width = widget->allocation.width;
   *height = widget->allocation.height;
@@ -668,7 +739,7 @@ gail_widget_get_size (AtkComponent   *component,
      */
     return;
 
-  g_return_if_fail (GTK_IS_WIDGET (widget));
+  gail_return_if_fail (GTK_IS_WIDGET (widget));
 
   *width = widget->allocation.width;
   *height = widget->allocation.height;
@@ -689,7 +760,7 @@ gail_widget_grab_focus (AtkComponent   *component)
   GtkWidget *widget = GTK_ACCESSIBLE (component)->widget;
   GtkWidget *toplevel;
 
-  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
+  gail_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
   if (GTK_WIDGET_CAN_FOCUS (widget))
     {
       gtk_widget_grab_focus (widget);
@@ -724,7 +795,7 @@ gail_widget_set_extents (AtkComponent   *component,
      * Object is defunct
      */
     return FALSE;
-  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
+  gail_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
   if (GTK_WIDGET_TOPLEVEL (widget))
     {
@@ -768,7 +839,7 @@ gail_widget_set_position (AtkComponent   *component,
      * Object is defunct
      */
     return FALSE;
-  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
+  gail_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
   if (GTK_WIDGET_TOPLEVEL (widget))
     {
@@ -809,7 +880,7 @@ gail_widget_set_size (AtkComponent   *component,
      * Object is defunct
      */
     return FALSE;
-  g_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
+  gail_return_val_if_fail (GTK_IS_WIDGET (widget), FALSE);
 
   if (GTK_WIDGET_TOPLEVEL (widget))
     {
@@ -820,26 +891,6 @@ gail_widget_set_size (AtkComponent   *component,
    return FALSE;
 }
 
-#if 0
-static GtkWidget*
-_gail_widget_get_parent (GtkWidget* widget)
-{
-  GtkWidget *parent_widget;
-  gpointer pointer;
-  GValue value = { 0, };
-
-  g_return_val_if_fail (GTK_WIDGET (widget), NULL);
-
-  g_value_init (&value, G_TYPE_OBJECT);
-  g_object_get_property (G_OBJECT (widget), "parent", &value);
-  pointer = g_value_get_object (&value);
- 
-  g_return_val_if_fail (GTK_IS_WIDGET (pointer), NULL);
-
-  return GTK_WIDGET (pointer);
-}
-#endif
-
 /*
  * This function is a signal handler for notify_in_event and focus_out_event
  * signal which gets emitted on a GtkWidget.
@@ -878,6 +929,24 @@ gail_widget_real_focus_gtk (GtkWidget     *widget,
   return FALSE;
 }
 
+static void
+gail_widget_size_allocate_gtk (GtkWidget     *widget,
+                               GtkAllocation *allocation)
+{
+  AtkObject* accessible;
+  AtkRectangle rect;
+
+  accessible = gtk_widget_get_accessible (widget);
+  if (ATK_IS_COMPONENT (accessible))
+    {
+      rect.x = allocation->x;
+      rect.y = allocation->y;
+      rect.width = allocation->width;
+      rect.height = allocation->height;
+      g_signal_emit_by_name (accessible, "bounds_changed", &rect);
+    }
+}
+
 /*
  * This function is the signal handler defined for map and unmap signals.
  */
@@ -954,7 +1023,12 @@ static void
 gail_widget_focus_event (AtkObject   *obj,
                          gboolean    focus_in)
 {
-  atk_object_notify_state_change (obj, ATK_STATE_FOCUSED, focus_in);
+  AtkObject *focus_obj;
+
+  focus_obj = g_object_get_data (G_OBJECT (obj), "gail-focus-object");
+  if (focus_obj == NULL)
+    focus_obj = obj;
+  atk_object_notify_state_change (focus_obj, ATK_STATE_FOCUSED, focus_in);
 }
 
 static GtkWidget*
