@@ -21,20 +21,24 @@
 #include <gtk/gtk.h>
 #include "gailmisc.h"
 
+/* IMPORTANT!!! This source file does NOT contain the implementation
+ * code for AtkUtil - for that code, please see gail/gail.c.
+ */
+
 /**
  * gail_misc_get_extents_from_pango_rectangle:
- * @widget: the widget that contains the PangoLayout, that contains
+ * @widget: The widget that contains the PangoLayout, that contains
  *   the PangoRectangle
- * @char_rect: the #PangoRectangle from which to calculate extents
- * @x_layout: the x-offset at which the widget displays the
+ * @char_rect: The #PangoRectangle from which to calculate extents
+ * @x_layout: The x-offset at which the widget displays the
  *   PangoLayout that contains the PangoRectangle, relative to @widget
- * @y_layout: the y-offset at which the widget displays the
+ * @y_layout: The y-offset at which the widget displays the
  *   PangoLayout that contains the PangoRectangle, relative to @widget
- * @x: the x-position of the #PangoRectangle relative to @coords
- * @y: the y-position of the #PangoRectangle relative to @coords
- * @width: the width of the #PangoRectangle
- * @height: the  height of the #PangoRectangle
- * @coords: an #AtkCoordType enumeration
+ * @x: The x-position of the #PangoRectangle relative to @coords
+ * @y: The y-position of the #PangoRectangle relative to @coords
+ * @width: The width of the #PangoRectangle
+ * @height: The  height of the #PangoRectangle
+ * @coords: An #AtkCoordType enumeration
  *
  * Gets the extents of @char_rect in device coordinates,
  * relative to either top-level window or screen coordinates as
@@ -56,17 +60,14 @@ gail_misc_get_extents_from_pango_rectangle (GtkWidget      *widget,
   gail_misc_get_origins (widget, &x_window, &y_window, 
                          &x_toplevel, &y_toplevel);
 
+  *x = (char_rect->x / PANGO_SCALE) + x_layout + x_window;
+  *y = (char_rect->y / PANGO_SCALE) + y_layout + y_window;
   if (coords == ATK_XY_WINDOW)
     {
-      *x = (char_rect->x / PANGO_SCALE) + x_layout + (x_window - x_toplevel);
-      *y = (char_rect->y / PANGO_SCALE) + y_layout + (y_window - y_toplevel);
+      *x -= x_toplevel;
+      *y -= y_toplevel;
     }
-  else if (coords == ATK_XY_SCREEN)
-    {
-      *x = (char_rect->x / PANGO_SCALE) + x_layout + x_window;
-      *y = (char_rect->y / PANGO_SCALE) + y_layout + y_window;
-    }
-  else
+  else if (coords != ATK_XY_SCREEN)
     {
       *x = 0;
       *y = 0;
@@ -82,18 +83,18 @@ gail_misc_get_extents_from_pango_rectangle (GtkWidget      *widget,
 
 /**
  * gail_misc_get_index_at_point_in_layout:
- * @widget: a #GtkWidget
- * @layout: the #PangoLayout from which to get the index at the
+ * @widget: A #GtkWidget
+ * @layout: The #PangoLayout from which to get the index at the
  *   specified point.
- * @x_layout: the x-offset at which the widget displays the
+ * @x_layout: The x-offset at which the widget displays the
  *   #PangoLayout, relative to @widget
- * @y_layout: the y-offset at which the widget displays the
+ * @y_layout: The y-offset at which the widget displays the
  *   #PangoLayout, relative to @widget
- * @x: the x-coordinate relative to @coords at which to
+ * @x: The x-coordinate relative to @coords at which to
  *   calculate the index
- * @y: the y-coordinate relative to @coords at which to
+ * @y: The y-coordinate relative to @coords at which to
  *   calculate the index
- * @coords: an #AtkCoordType enumeration
+ * @coords: An #AtkCoordType enumeration
  *
  * Gets the byte offset at the specified @x and @y in a #PangoLayout.
  *
@@ -110,29 +111,39 @@ gail_misc_get_index_at_point_in_layout (GtkWidget   *widget,
                                         AtkCoordType coords)
 {
   gint index, x_window, y_window, x_toplevel, y_toplevel;
+  gint x_temp, y_temp;
+  gboolean ret;
 
   gail_misc_get_origins (widget, &x_window, &y_window, 
                          &x_toplevel, &y_toplevel);
+  x_temp =  x - x_layout - x_window;
+  y_temp =  y - y_layout - y_window;
   if (coords == ATK_XY_WINDOW)
-    pango_layout_xy_to_index (layout,
-      (x - x_layout - (x_window - x_toplevel)) * PANGO_SCALE,
-      (y - y_layout - (y_window - y_toplevel)) * PANGO_SCALE,
-      &index, NULL);
-  else if (coords == ATK_XY_SCREEN)
-    pango_layout_xy_to_index (layout,
-      (x - x_layout - x_window) * PANGO_SCALE,
-      (y - y_layout - y_window) * PANGO_SCALE,
-      &index, NULL);
-  else
+    {
+      x_temp += x_toplevel;  
+      y_temp += y_toplevel;
+    }
+  else if (coords != ATK_XY_SCREEN)
     return -1;
 
+  ret = pango_layout_xy_to_index (layout, 
+                                  x_temp * PANGO_SCALE,
+                                  y_temp * PANGO_SCALE,
+                                  &index, NULL);
+  if (!ret)
+    {
+      if (x_temp < 0 || y_temp < 0)
+        index = 0;
+      else
+        index = -1; 
+    }
   return index;
 }
 
 /**
  * gail_misc_add_attribute:
  * @attrib_set: The #AtkAttributeSet to add the attribute to
- * @attr: The AtkTextAttrribute which identifies the attribut
+ * @attr: The AtkTextAttrribute which identifies the attribute to be added
  * @value: The attribute value
  *
  * Creates an #AtkAttribute from @attr and @value, and adds it
@@ -521,23 +532,27 @@ gail_misc_get_origins (GtkWidget *widget,
 {
   GdkWindow *window;
 
-  gdk_window_get_origin (widget->window, x_window, y_window);
+  if (GTK_IS_TREE_VIEW (widget))
+    window = gtk_tree_view_get_bin_window (GTK_TREE_VIEW (widget));
+  else
+    window = widget->window;
+  gdk_window_get_origin (window, x_window, y_window);
   window = gdk_window_get_toplevel (widget->window);
   gdk_window_get_origin (window, x_toplevel, y_toplevel);
 }
 
 /**
  * gail_misc_add_to_attr_set:
- * @attrib_set: a #AtkAttributeSet
- * @attrs: the #GtkTextAttributes containing the attribute value
- * @attr: the #AtkTextAttribute to be added
- *
- * Returns: A pointer to the #AtkAttributeSet.
+ * @attrib_set: An #AtkAttributeSet
+ * @attrs: The #GtkTextAttributes containing the attribute value
+ * @attr: The #AtkTextAttribute to be added
  *
  * Gets the value for the AtkTextAttribute from the GtkTextAttributes
- * and adds it to the the AttributeSet.
+ * and adds it to the AttributeSet.
+ *
+ * Returns: A pointer to the updated #AtkAttributeSet.
  **/
-AtkAttributeSet *
+AtkAttributeSet*
 gail_misc_add_to_attr_set (AtkAttributeSet   *attrib_set,
                            GtkTextAttributes *attrs,
                            AtkTextAttribute  attr)
